#!/bin/sh
#------------------------------------------------------------------------------
# /tmp/install.sh - installation file for nfsserver services on EIS/FAIR
#
# Copyright (c) 2001-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2002-10-09 M.Schimanski
# Last Update:  $Id: install.sh 73445 2020-03-24 11:54:19Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

module_name='nfsserver'

installfile=/var/run/${module_name}.install

myecho --pack ${module_name} --info "running install.sh ..."

# update shared library file
update_jedlib ${module_name}

# file names and pathes
### symbolic links
myecho --pack ${module_name} "creating symbolic links ..."

rm -f /etc/rc2.d/[KS]??${module_name}

ln -sf /etc/init.d/${module_name} /etc/rc2.d/K55${module_name}
ln -sf /etc/init.d/${module_name} /etc/rc2.d/S45${module_name}

# add menu entry
myecho --pack ${module_name} "adding menu  ..."
/var/install/bin/add-menu --menu "setup.services.menu" "setup.services.${module_name}.menu" "NFSserver Service"

# creating files and directories
for DNAME in '' rpcbind nfs nfs/sm nfs/sm.bak nfs/v4recovery nfs/rpc_pipefs nfs/nfsdcltrack
do
    if [ ! -d /var/lib/${DNAME} ]
    then
        mkdir -p /var/lib/${DNAME}
    fi

    chmod 0755 /var/lib/${DNAME}
done

for FNAME in etab rmtab xtab
do
    # create empty files
    > /var/lib/nfs/$FNAME

    chmod 0644 /var/lib/nfs/$FNAME
done

# change ownership to supress rpc.statd message
chown -R nobody  /var/lib/nfs
chgrp -R nogroup /var/lib/nfs

# change ownership to suppress nfsdcltrack message
chown -R root /var/lib/nfs/nfsdcltrack
chgrp -R root /var/lib/nfs/nfsdcltrack
chmod 0700 /var/lib/nfs/nfsdcltrack

if [ ! -f /var/lib/nfs/nfsdcltrack/main.sqlite ]
then
    # create an initial database file
    /sbin/nfsdcltrack init
fi

# install/update package configuration file
silent=''
if called_from_install --pack "${module_name}"
then
    silent='--quiet'
fi

update=false
if install_config ${silent} ${module_name}
then
    update=true
fi

# updated installation
/var/install/config.d/${module_name}.sh

if [ -f /etc/init.d/${module_name} ]
then
    /etc/init.d/${module_name} restart
fi

exit 0
