#!/bin/sh
#------------------------------------------------------------------------------
# /tmp/preinstall.sh - pre installation file for nfsserver services on EIS/FAIR
#
# Copyright (c) 2001-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2002-10-09 M.Schimanski
# Last Update:  $Id: preinstall.sh 73445 2020-03-24 11:54:19Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# check if ip address
# input : $1 - address
# return:  0 - is ip-address
#          1 - no ip-address
#------------------------------------------------------------------------------
is_ipaddr ()
{
    local _ii_addr

    _ii_addr=`echo $1 | tr '.' ' '`

    for NUM in ${_ii_addr}
    do
        if ! is_numeric ${NUM}
        then
            return 1
        fi
    done

    return 0
}

#------------------------------------------------------------------------------
# complete network address or netmask
# input : $1 - addr|mask
# output: $2 - network
#------------------------------------------------------------------------------
complete_network_netmask ()
{
    local _cnn_type=$1
    local _cnn_network=`echo $2 | tr '.' ' '`
    local _cnn_outnet
    local _cnn_outmask
    local _cnn_idx

    _cnn_outnet=''
    _cnn_outmask=''

    _cnn_idx=0
    for FIELD in ${_cnn_network}
    do
        if [ -z "${_cnn_outnet}" ]
        then
            _cnn_outnet="${FIELD}"
            _cnn_outmask="255"
        else
            _cnn_outnet="${_cnn_outnet}.${FIELD}"
            _cnn_outmask="${_cnn_outmask}.255"
        fi

        _cnn_idx=`expr ${_cnn_idx} + 1`
    done

    if [ ${_cnn_idx} -lt 4 ]
    then
        while [ ${_cnn_idx} -lt 4 ]
        do
            _cnn_outnet="${_cnn_outnet}.0"
            _cnn_outmask="${_cnn_outmask}.0"

            _cnn_idx=`expr ${_cnn_idx} + 1`
        done
    fi

    case ${_cnn_type} in
        addr )
            echo "${_cnn_outnet}"
            ;;
        mask )
            echo "${_cnn_outmask}"
            ;;
    esac
}

#------------------------------------------------------------------------------
# read exports file
#------------------------------------------------------------------------------
read_exports_file ()
{
    local _ref_exportsfile=/etc/exports
    local _ref_outfile=${configfile}
    local _ref_idx
    local _ref_line
    local _ref_nfs_path
    local _ref_nfs_tmp_str
    local _ref_jdx
    local _ref_exitflag
    local _ref_nfs_host_net
    local _ref_nfs_options
    local _ref_nfs_host
    local _ref_nfs_net
    local _ref_prefix
    local _ref_nfs_rw
    local _ref_nfs_uid
    local _ref_nfs_gid
    local _ref_nfs_root_squash
    local _ref_nfs_all_squash
    local _ref_nfs_map_daemon

    if [ -f ${_ref_exportsfile} -a ! -f ${_ref_outfile} ]
    then
        {
            _ref_idx=0
            while read LINE
            do
                _ref_line=`echo ${LINE}`

                if [ -n "${_ref_line}" ]
                then
                    echo ${_ref_line} | grep -q "^#"

                    if [ $? -ne 0 ]
                    then
                      # echo ">>>>>|${_ref_line}|"

                        # analyse command
                        _ref_idx=`expr ${_ref_idx} + 1`

                        # print separator
                        if [ ${_ref_idx} -gt 1 ]
                        then
                            echo '#------------------------------------------------------------------------------'
                        fi

                        # get share path
                        _ref_nfs_path=`echo ${_ref_line} | cut -d' ' -f1`
                        _ref_nfs_tmp_str=`echo ${_ref_line} | cut -d' ' -f2-`

                        # output
                        if [ ${_ref_idx} -eq 1 ]
                        then
                            echo "NFSSERVER_SHARE_N='xxx'"
                        fi
                        echo "NFSSERVER_SHARE_${_ref_idx}_PATH='${_ref_nfs_path}'"

                        _ref_jdx=1
                        _ref_exitflag=0
                        while [ ${_ref_exitflag} -eq 0 ]
                        do
                            _ref_nfs_host_net=`echo ${_ref_nfs_tmp_str} | cut -d'(' -f1`
                            _ref_nfs_options=`echo ${_ref_nfs_tmp_str} | cut -d'(' -f2|cut -d')' -f1`
                            _ref_nfs_tmp_str=`echo ${_ref_nfs_tmp_str} | cut -d')' -f2-`

                          # echo ">>>>>|${_ref_nfs_tmp_str}|"

                            # check host and netmask
                            echo "${_ref_nfs_host_net}" | grep -q "/"

                            if [ $? -eq 0 ]
                            then
                                # split into parts
                                _ref_nfs_host=`echo "${_ref_nfs_host_net}" | cut -d'/' -f1`
                                _ref_nfs_host=`complete_network_netmask addr ${_ref_nfs_host}`

                                _ref_nfs_net=`echo "${_ref_nfs_host_net}" | cut -d'/' -f2`

                                # concat only if network and not ip-address
                                _ref_prefix=`netcalc netmaskbits ${_ref_nfs_net}`

                                if [ ${_ref_prefix} -lt 32 ]
                                then
                                    # concat parts
                                    _ref_nfs_host="${_ref_nfs_host}/`netcalc netmaskbits ${_ref_nfs_net}`"
                                fi
                            else
                                # only ip-address given
                                if is_ipaddr ${_ref_nfs_host_net}
                                then
                                    # complete address and mask
                                    _ref_nfs_host=`complete_network_netmask addr ${_ref_nfs_host_net}`
                                    _ref_nfs_net=`complete_network_netmask mask ${_ref_nfs_host_net}`

                                    # concat only if network and not ip-address
                                    _ref_prefix=`netcalc netmaskbits ${_ref_nfs_net}`

                                    if [ ${_ref_prefix} -lt 32 ]
                                    then
                                        # concat parts
                                        _ref_nfs_host="${_ref_nfs_host}/`netcalc netmaskbits ${_ref_nfs_net}`"
                                    fi
                                else
                                    # take as hostname
                                    _ref_nfs_host="${_ref_nfs_host_net}"
                                fi
                            fi

                            # check access
                            echo "${_ref_nfs_options}" | grep -q "rw"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_rw='yes'
                            else
                                _ref_nfs_rw='no'
                            fi

                            # check anonuid
                            echo "${_ref_nfs_options}" | grep -q "anonuid"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_uid=`echo "${_ref_nfs_options}" | sed -e 's/.*anonuid.*=//g' -e 's/[, ].*$//g'`
                            else
                                _ref_nfs_uid=''
                            fi

                            # check anongid
                            echo "${_ref_nfs_options}" | grep -q "anongid"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_gid=`echo "${_ref_nfs_options}" | sed -e 's/.*anongid.*=//g' -e 's/[, ].*$//g'`
                            else
                                _ref_nfs_gid=''
                            fi

                            # check no_root_squash
                            echo "${_ref_nfs_options}" | grep -q "no_root_squash"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_root_squash='no'
                            else
                                _ref_nfs_root_squash='yes'
                            fi

                            # check all_squash
                            echo "${_ref_nfs_options}" | grep -q "all_squash"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_all_squash='yes'
                            else
                                _ref_nfs_all_squash='no'
                            fi

                            # check map_daemon
                            echo "${_ref_nfs_options}" | grep -q "map_daemon"

                            if [ $? -eq 0 ]
                            then
                                _ref_nfs_map_daemon='yes'
                            else
                                _ref_nfs_map_daemon='no'
                            fi

                            # output
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_HOST='${_ref_nfs_host}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_RW='${_ref_nfs_rw}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_ANONUSER='${_ref_nfs_uid}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_ANONGROUP='${_ref_nfs_gid}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_ROOT_SQUASH='${_ref_nfs_root_squash}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_ALL_SQUASH='${_ref_nfs_all_squash}'"
                            echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_${_ref_jdx}_MAP_DAEMON='${_ref_nfs_map_daemon}'"

                            if [ -n "${_ref_nfs_tmp_str}" ]
                            then
                                _ref_jdx=`expr ${_ref_jdx} + 1`
                            else
                                # update host counter
                                echo "NFSSERVER_SHARE_${_ref_idx}_HOSTS_N='${_ref_jdx}'"

                                _ref_exitflag=1
                            fi
                        done
                    fi
                fi
            done < ${_ref_exportsfile}
        } > ${_ref_outfile}.tmp

        # update share counter
        sed "s/NFSSERVER_SHARE_N='xxx'/NFSSERVER_SHARE_N='${_ref_idx}'/g" ${_ref_outfile}.tmp > ${_ref_outfile}
        rm -f ${_ref_outfile}.tmp
    fi
}

#==============================================================================
# main
#==============================================================================

module_name='nfsserver'

configfile=/etc/config.d/${module_name}
filelist=/etc/filelist.d/${module_name}-files.txt
installfile=/var/run/${module_name}.install

# extract shared library file from archive
# to be able to use the myecho command
archivefile=`find /tmp -maxdepth 1 -type l -name "${module_name}.tar.*" ! -name "*.info" | head -n1`

if [ -n "${archivefile}" ]
then
    if [ -s "${archivefile}" ]
    then
        tar xjf ${archivefile} var/install/include/jedlib.${module_name} -C /

        srcfile=/var/install/include/jedlib.${module_name}
        destfile=/var/install/include/jedlib

        if [ -f ${srcfile} ]
        then
            if [ -f ${destfile} ]
            then
                srcver=`/bin/sh ${srcfile} version`
                destver=`/bin/sh ${destfile} version`

                if [ -n "${destver}" ]
                then
                    # version number exists - compare version numbers
                    if [ `echo "${srcver} > ${destver}" | bc` -eq 1 ]
                    then
                        # update file
                        echo "updating shared library file (${destver}->${srcver}) ..."
                        mv -v ${srcfile} ${destfile}
                    fi
                else
                    # no version number exists - update file
                    echo "updating shared library file ..."
                    mv -v ${srcfile} ${destfile}
                fi
            else
                # file doesn't exist - update file
                echo "updating shared library file ..."
                mv -v ${srcfile} ${destfile}
            fi
        fi
    fi
fi

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

myecho --pack ${module_name} --info "running preinstall.sh ..."

# check nfsserver version
installed_version=`/usr/bin/eisman check nfsserver 1.0.0`

case $installed_version in
    installed|new )
        # old version without complex configuration file
        # trying to import information from /etc/exports
        myecho --pack ${module_name} "old nfsserver package found, trying to import data from exports file ..."
        myecho --pack ${module_name}

        # backup old file
        /var/install/bin/backup-file --quiet /etc/exports backup

        myecho --pack ${module_name} "  The original file has been saved as /etc/exports.backup."
        myecho --pack ${module_name} "  Importing /etc/exports ..."

        read_exports_file

        myecho --pack ${module_name} --info "  Finished, please verify that everything has been imported properly!"
        ;;
    * ) # not-installed / old
        ;;
esac

# remove mecho
installed_version=`/usr/bin/eisman check nfsclient 1.0.5`

case $installed_version in
    not-installed|old ) # remove mecho script
        rm -f /usr/local/bin/mecho
        ;;
    * ) # installed / new
        ;;
esac

# since v1.5.0 the program sm-notify is only part of the nfsclient
# pakage. Make sure that it isn't removed during an update of the
# nfsserver package.
if [ -f ${filelist} ]
then
    grep -q " usr/sbin/sm-notify" ${filelist}

    if [ $? -eq 0 ]
    then
        cp  ${filelist} ${filelist}.tmp
        grep -v " usr/sbin/sm-notify" ${filelist}.tmp > ${filelist}
        rm -f ${filelist}.tmp
    fi
fi

# backup existing file
if [ -f ${configfile} ]
then
    # move old files to backup directory and create backup
    /var/install/bin/backup-file --quiet ${module_name}

    myecho --pack ${module_name} "saving previous configuration file to ${installfile} ..."
    cp ${configfile} ${installfile}
fi

# deinstall old software
myecho --pack ${module_name} "deinstalling previously installed software ..."
/usr/bin/eisman uninstall --update --auto --quiet ${module_name}

exit 0
